package com.xy.alipay;

import cn.hutool.json.JSONObject;
import com.xy.annotate.RestMappingController;
import com.xy.config.DeviceThreadPoolConfig;
import com.xy.consumer.connected.ConnectedMqttConfiguration;
import com.xy.consumer.connected.ConnectedProducer;
import com.xy.consumer.disconnect.DisconnectedMqttConfiguration;
import com.xy.consumer.disconnect.DisconnectedProducer;
import com.xy.dto.DeviceDetailDTO;
import com.xy.service.AlipayDeviceService;
import com.xy.utils.R;
import com.xy.utils.ThreadPoolUtils;
import com.xy.vo.DeviceDetailVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * <p>
 * 支付宝设备服务
 * </p>
 *
 * @author lijin
 * @since 2023-04-20
 */
@Slf4j
@Component
@AllArgsConstructor
@Api(tags = "支付宝设备API")
@RestMappingController("/api/device")
public class ApiDeviceServiceImpl {

    private AlipayDeviceService alipayDeviceService;

    private ConnectedProducer connectedProducer;

    private DisconnectedProducer disconnectedProducer;

    @ApiOperation("刷新设备联网状态")
    @PostMapping("refurbishDeviceNetWork")
    public R refurbishDeviceNetWork(@RequestBody List<Long> deviceIds) {
        ThreadPoolUtils.Execute execute = ThreadPoolUtils.excPoll(DeviceThreadPoolConfig.ALIPAY_DEVICE_DETAIL, deviceIds.size());
        deviceIds.forEach(deviceId -> execute.execute(() -> {
            DeviceDetailVO deviceDetailVO = alipayDeviceService.queryDetail(new DeviceDetailDTO().setTerminalId(String.valueOf(deviceId)));
            if (deviceDetailVO == null) {
                return;
            }
            JSONObject jsonObject = new JSONObject().set("clientid", deviceId);
            if (deviceDetailVO.getOnline()) {
                connectedProducer.sendToMqtt(jsonObject.toString(), ConnectedMqttConfiguration.TOPIC, 1);
            } else {
                disconnectedProducer.sendToMqtt(jsonObject.toString(), DisconnectedMqttConfiguration.TOPIC, 1);
            }
        }));
        execute.end();
        return R.ok();
    }


}
